@@ -117,7 +117,8 @@ class IrcCallback(IrcCommandDispatcher, log.Firewalled):
     """
     callAfter = ()
     callBefore = ()
-    echo_message = False
+    echoMessage = False
+    echo_message = False  # deprecated alias of echoMessage
     __firewalled__ = {'die': None,
                       'reset': None,
                       '__call__': None,
@@ -182,7 +183,7 @@ class IrcCallback(IrcCommandDispatcher, log.Firewalled):
 
     def __call__(self, irc, msg):
         """Used for handling each message."""
-        if not self.echo_message \
+        if not self.echoMessage and not self.echo_message \
                 and msg.command in ('PRIVMSG', 'NOTICE', 'TAGMSG') \
                 and ('label' in msg.server_tags
                      or not msg.tagged('receivedAt')):
@@ -624,9 +625,14 @@ class IrcState(IrcCommandDispatcher, log.Firewalled):
         Supported user and channel modes are cached"""
         # msg.args = [nick, server, ircd-version, umodes, modes,
         #             modes that require arguments? (non-standard)]
-        self.ircd = msg.args[2] if len(msg.args) >= 3 else msg.args[1]
-        self.supported['umodes'] = frozenset(msg.args[3])
-        self.supported['chanmodes'] = frozenset(msg.args[4])
+        self.ircd = msg.args[2] if len(msg.args) > 2 else msg.args[1]
+
+        # The conditionals are for Twitch, which doesn't send umodes or
+        # chanmodes.
+        if len(msg.args) > 3:
+            self.supported['umodes'] = frozenset(msg.args[3])
+        if len(msg.args) > 4:
+            self.supported['chanmodes'] = frozenset(msg.args[4])
 
     _005converters = utils.InsensitivePreservingDict({
         'modes': lambda s: int(s) if s else None,  # it's optional
